/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: flag.h,v 1.2 2003/07/28 11:28:51 jaffe Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jaffe 2003-07-24  written
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
/* $Id: flag.h,v 1.2 2003/07/28 11:28:51 jaffe Exp $ */

#ifndef _FLAG_TABLE_H
#define _FLAG_TABLE_H
/*----------------------------------------------------------------------- */
/* Flag table FITS binary tables routines                                */
/*----------------------------------------------------------------------- */
#include "VLTItable.h"

typedef struct      /* table definition for flag table */
{
  table_def *def;       /* basic table information */
  long   row;           /* last row (0-rel) read or written, -1=> none. */
  char   *date_obs;     /* observing date */
  int    noChan;        /* number of spectral channels*/
  int    timeRange_col;      /* Time Range column number (0-rel) */
  int    source_col;      /* source number column number (0-rel) */
  int    telescope_col; /* Telescope number column number (0-rel) */
  int    chans_col;     /* flag column number */
  int    reason_col;    /* flag column number */
} flag_table_def;

typedef struct      /* structure used for jaffe row */
{
  double     timeRange[2];     /* time (days) */
  int        sourceId; /* which source entry */
  int        telescope[2]; /* which telescope entry*/
  int        chans[2]; /* which spectral channels */
  char       reason[26];    /* why? */
  table_row *raw;    /* generic table row */
} flag_table_row;

/* public functions */

/* flag_table_row functions */
flag_table_row* make_flag_table_row(flag_table_def* table);
/*----------------------------------------------------------------------- */
/*   Make an flag_table_row structure                           */
/*   Inputs:                                                              */
/*      table     pointer to flag_table table structure         */
/*   Returns:                                                             */
/*      An flag_table_row structure pointer ; NULL => failure   */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */

void kill_flag_table_row (flag_table_row *row);
/*----------------------------------------------------------------------- */
/*   Delete an flag_table_row structure                         */
/*   Inputs:                                                              */
/*      row    pointer to the flag_table_row structure.         */
/*----------------------------------------------------------------------- */

/* flag_table functions */

flag_table_def* 
make_flag_table_def(fitsfile *fptr, char *date_obs, int noChan, int *ierr);
/*----------------------------------------------------------------------- */
/*   Make a new flag_table in an open FITS file                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      date_obs  Date (yyyy-mm-dd) observations began                    */
/*      noChan    number of spectral channels in observatins              */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      A flag_table_def pointer; NULL indicates failure        */
/*----------------------------------------------------------------------- */

flag_table_def* 
open_flag_table(fitsfile *fptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Open an existing flag_table in the active FITS file        */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An flag_table_def pointer; NULL indicates failure       */
/*----------------------------------------------------------------------- */

int read_flag_table(fitsfile *fptr,  flag_table_def* table, 
			    flag_table_row *row, long row_number,
			    int *ierr);
/*----------------------------------------------------------------------- */
/*   Read the next row from an flag_table                       */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int write_flag_table(fitsfile *fptr, flag_table_def *table, 
			     flag_table_row *row,  long row_number,
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write  the next row to an flag_table                               */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int close_flag_table(fitsfile *fptr, flag_table_def* table, 
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Close an open flag_table, associated structures are        */
/*   destroyed                                                            */
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

void kill_flag_table_def (flag_table_def* table);
/*----------------------------------------------------------------------- */
/*   Delete an flag_table structure                             */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */

#endif /* end _FLAG_TABLE_H */
/* */
